use super::*;
use crate::parse::{
    element::test::{from_html_str, grab_element_from_document},
    ElementRef,
};

#[test]
fn url() -> Result<(), crate::Error> {
    let dom = from_html_str(r#"<div class="h-item"><a class="h-item" href="/"></a></div>"#);
    let parent_elem = grab_element_from_document(&dom, "div").unwrap();

    let parent_parser = PropertiesParser::new(
        Arc::new(ElementRef {
            index: 0,
            node: Node { elem: parent_elem },
        }),
        Default::default(),
        &"http://example.com".parse()?,
    );

    assert!(
        parent_parser.can_imply_url(),
        "can expect to get an implied URL"
    );

    assert_eq!(parent_parser.implied_url(), None, "did not imply a URL");

    let child_elem = grab_element_from_document(&dom, "a").unwrap();

    let child_parser = PropertiesParser::new(
        Arc::new(ElementRef {
            index: 0,
            node: Node { elem: child_elem },
        }),
        Default::default(),
        &"http://example.com".parse()?,
    );

    assert!(
        child_parser.can_imply_url(),
        "can expect to get an implied URL"
    );

    assert_eq!(
        child_parser.implied_url(),
        Some(PropertyValue::Url("http://example.com/".parse()?)),
        "implied a URL"
    );

    Ok(())
}
